/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert Analysedaten fr die Situationsbestimmung pro Umfelddatenmessstelle
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class SituationAnalyseUfdms
{
	/**
	 * Status Einschaltkriterium.
	 */
	private boolean m_einschaltkriterium;
	/**
	 * Status Ausschaltkriterium.
	 */
	private boolean m_ausschaltkriterium;
  /**
   * Status der Situationsbestimmung pro Umfelddatenmessstelle
   */
  private List<Status> m_statusUfdms = new ArrayList<Status>();
  
  
  /**
   * Default-Konstruktor ohne weitere Funktion.
   */
  public SituationAnalyseUfdms()
  {
  }
  
	/**
	 * Konstruktor mit bernahme der Daten.
	 * 
	 * @param rdata zu bernehmender Datensatz.
	 */
	public SituationAnalyseUfdms(ResultData rdata)
	{
		if (rdata != null)
		{
			if (rdata.hasData())
			{
				Data data = rdata.getData();
				
				m_einschaltkriterium = data.getScaledValue("Einschaltkriterium").getValueText().equals("Ja");
				m_ausschaltkriterium = data.getScaledValue("Ausschaltkriterium").getValueText().equals("Ja");

				for (int i = 0; i < data.getArray("Status").getLength(); i++)
				{					
					Data element = data.getArray("Status").getItem(i);
					if (element != null)
					{
						addAnalyseUfdms(element.getReferenceValue("UmfeldDatenMessStelle").getSystemObject(), element.getScaledValue("Aktiv").getValueText().equals("Ja"));
					}			
				}
			}
		}
	}

	/**
	 * Fgt zu der Liste der einen neuen Datensatz mit den bergebenen Werten hinzu.
	 * 
	 * @param ufdms Systemobjekt Umfelddatenmessstelle.
	 * @param status Status der Situation;
	 */
	public void addAnalyseUfdms(SystemObject ufdms, boolean status)
	{
		if (m_statusUfdms == null)
		{
			m_statusUfdms = new ArrayList<Status>();;
		}
		
		Status werte = new Status();
		werte.setUmfeldDatenMessstelle(ufdms);
		werte.setAktiv(status);
		
		m_statusUfdms.add(werte);
	}

  /**
   * Klasse speichert die Zwischenergebnisse fr eine Umfelddatenmessstelle
   * @author Swarco Traffic Systems GmbH
   * @author Matthias Obert
   *
   */
  private class Status
  {
    /**
     * Umfelddatenmessstelle
     */
    private SystemObject m_umfeldDatenMessstelle;
    
    /**
     * Aktueller Status der Situationsbestimmung (aktiv/inaktiv).
     */
    private boolean m_aktiv;
   
    /**
     * Parameterloser Konstruktor der Klasse
     */
    private Status ()
    {
      
    }

    
    /**
     * Methode liefert die Variable m_umfeldDatenMessstelle zurck
     * @return Variable m_umfeldDatenMessstelle
     */
    public SystemObject getUmfeldDatenMessstelle()
    {
      return m_umfeldDatenMessstelle;
    }


    /**
     * Methode setzt die Variable m_umfeldDatenMessstelle
     * @param umfeldDatenMessstelle Wert auf den die Variable m_umfeldDatenMessstelle gesetzt werden soll
     */
    public void setUmfeldDatenMessstelle( SystemObject umfeldDatenMessstelle )
    {
      m_umfeldDatenMessstelle = umfeldDatenMessstelle;
    }


    /**
     * Methode liefert die Variable m_aktiv zurck
     * @return Variable m_aktiv
     */
    public boolean isAktiv()
    {
      return m_aktiv;
    }

    /**
     * Methode setzt die Variable m_aktiv
     * @param aktiv Wert auf den die Variable m_aktiv gesetzt werden soll
     */
    public void setAktiv( boolean aktiv )
    {
      m_aktiv = aktiv;
    }
  }

  /**
   * Methode liefert die Variable m_statusUfdms zurck
   * @return Variable m_statusUfdms
   */
  public List<Status> getStatusUfdms()
  {
    return m_statusUfdms;
  }

  /**
   * Methode setzt die Variable m_statusUfdms
   * @param statusUfdms Wert auf den die Variable m_statusUfdms gesetzt werden soll
   */
  public void setStatusUfdms( List<Status> statusUfdms )
  {
    m_statusUfdms = statusUfdms;
  }
	/**
	 * Prft den Status des Einschaltkriteriums.
	 * 
	 * @return true wenn Einschaltkriteriums wahr, ansonsten false.
	 */
	public boolean isEinschaltkriterium()
	{
		return m_einschaltkriterium;
	}

	/**
	 * Setzt den Status des Einschaltkriteriums.
	 * @param einschaltkriterium zu setzender Status.
	 */
	public void setEinschaltkriterium(boolean einschaltkriterium)
	{
		m_einschaltkriterium = einschaltkriterium;
	}

	/**
	 * Prft den Status des Ausschaltkriteriums.
	 * 
	 * @return true wenn Ausschaltkriteriums wahr, ansonsten false.
	 */
	public boolean isAusschaltkriterium()
	{
		return m_ausschaltkriterium;
	}

	/**
	 * Setzt den Status des Ausschaltkriteriums.
	 * @param ausschaltkriterium zu setzender Status.
	 */
	public void setAusschaltkriterium(boolean ausschaltkriterium)
	{
		m_ausschaltkriterium = ausschaltkriterium;
	}
}
