/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.nachlauf;

import static org.junit.Assert.fail;

import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.DefinitionenGlobal;
import de.bsvrz.ste.stealgsba.algorithmen.lib.EinzelsituationVorstufeEmpfang;
import de.bsvrz.ste.stealgsba.algorithmen.lib.INeueDatenListener;
import de.bsvrz.ste.stealgsba.algorithmen.lib.IUfdsNeueDatenListener;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SenderOhneStatusRueckmeldung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SimpleSenderMitWartenAufParametrierung;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseMq;
import de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseUfd;
import de.bsvrz.ste.stealgsba.algorithmen.lib.StaukriteriumErgebnis;
import de.bsvrz.ste.stealgsba.algorithmen.lib.TestTools;
import de.bsvrz.ste.stealgsba.algorithmen.nachlauf.ParameterNachlauf.Reaktion;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.AnzeigenDatensatz;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.EinzelMassnahmeAlternative;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.Massnahmenanforderung;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.losb.exceptions.SenderException;
import de.bsvrz.sys.funclib.vewdynobj.VerwaltungDynObj;

/**
 * Klasse fr die Durchfhrung der Prfflle 13.26 und 13.27 wie in der Prfspezifikation beschrieben.
 *
 * @version $Revision: 1.4 $ / $Date: 2015/09/10 11:24:16 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class NachlaufTest implements INeueDatenListener, IUfdsNeueDatenListener
{
  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface m_connection = null;
  /**
   * Aktuelles Datenmodell.
   */
  private static DataModel m_dataModel;  
  /**
   * Typ der Ursacheneinheit.
   */
  private static String m_typUe;
  /**
   * Pid der Ursacheneinheit.
   */
  private static String m_pidUe;
  /**
   * Name der Ursacheneinheit.
   */
  private static String m_nameUe;
  /**
   * Pid der Streckenbeeinflussungsanlage.
   */
  private static String m_anlage;
  /**
   * Synchronisierungsobjekt.
   */
  private static Object m_objekt;
  /**
   * Synchronisierungsobjekt.
   */
  private static Object m_objekt2;
  /**
   * Pid der Anzeige.
   */
  private static String m_pidAnzeige;
  /**
   * Datensatz Einzelsituationvorstufe.
   */
  private static ResultData m_DatensatzEinzelsituationVorstufe = null;
  /**
   * Zugriff auf die Instanz des Algorithmus.
   */
  private static NachlaufAlg m_alg;
  /**
   * Zugriff auf die Instanz der Verwaltung dynamischer Objekte.
   */
  private static VerwaltungDynObj m_vdo;  	
  /**
   * Flag Daten erhalten.
   */
  private static boolean m_datenErhalten = false;
  /**
   * PID Ursacheneinheit Input.
   */
	private static String m_pidUeNachlauf1;
  /**
   * Name Ursacheneinheit Input.
   */
	private static String m_nameUeNachlauf1;
  /**
   * Typ Ursacheneinheit Input.
   */
	private static String m_typUeNachlaufInput;
	
	public enum Prueffall
	{
		/**
		 * Prffall 1.
		 */
		PRF1,
		/**
		 * Prffall 2 Parametrierung.
		 */
		PRF2,
		/**
		 * Prffall 3 Parametrierung.
		 */
		PRF3;
	}

	/**
   * Initialisiert die Testumgebung.
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  	// Globale Daten setzen
    m_pidAnzeige = "bw.uz.B27.streckenBeeinflussungsAnlage.testAnzeigeQuerschnitt1.anzeige1";
//    m_pidAnzeige = "anzeige.A8.8351.HFB.O.anzeige1";

    m_pidUeNachlauf1 = "bw.uz.B27.UeNachlauf.input1";
    m_nameUeNachlauf1 = "UrsachenEinheitUeNachlauf Input1";
    m_typUeNachlaufInput = "typ.sswSbaUeHarmoBelastungUnruhe120";
    
    m_anlage = DefinitionenGlobal.getAnlage();
    m_typUe = "typ.sswSbaUeNachlauf";
    m_pidUe = "bw.uz.B27.UeNachlauf.JUnitTest1";
    m_nameUe = "UrsachenEinheitNachlauf.JUnitTest1";

    // Debug-Level setzen    
    Debug.setLoggerLevel("DEFAULT", Debug.OFF);
    Debug.setHandlerLevel("StdErr", Debug.OFF);
    
    // Datenverteilerverbindung aufbauen    
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress(DefinitionenGlobal.getDavCommunicationAddress());
    parameters.setDavCommunicationSubAddress(DefinitionenGlobal.getDavCommunicationSubAddress());
    parameters.setUserName(DefinitionenGlobal.getUserName());
    parameters.setUserPassword(DefinitionenGlobal.getUserPassword());

    m_connection = new ClientDavConnection(parameters);
    m_connection.connect();
    m_connection.login();
    m_dataModel = m_connection.getDataModel();
    
    // Synchronisierungsobjekte erstellen
    m_objekt = new Object();
    m_objekt2 = new Object();
    
    // Aufrufparameter definieren
    String[] argumente = 
    {
     	DefinitionenGlobal.getArgumentDav(),
     	DefinitionenGlobal.getArgumentBenutzer(),
     	DefinitionenGlobal.getArgumentDebugLevelStdErrText(),
     	DefinitionenGlobal.getArgumentAuthentifizierung(),
  		"-Anlage=" + DefinitionenGlobal.getAnlage(),
  		"-AspektVersand=asp.sswSbaNachlauf",
  		DefinitionenGlobal.getMaxZyklusDauer(),
  		DefinitionenGlobal.getZyklusLaenge(),
  		DefinitionenGlobal.getDeltaPruefen(),
  		"-Spontan=true"
    };
    		
		// Algorithmus instanziieren und starten
    m_alg = new NachlaufAlg();
    
    StandardApplicationRunner.run(m_alg, argumente);
    
    // UE erzeugen und in Menge legen            
    m_vdo = new VerwaltungDynObj();
    m_vdo.setConnection(m_connection);
    
    try
		{
    	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
			m_vdo.loescheObjekt(m_pidUe);

			m_vdo.reduziereMenge(m_pidUeNachlauf1, "UrsachenEinheiten", m_anlage);
			m_vdo.loescheObjekt(m_pidUeNachlauf1);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

    try
		{
			m_vdo.erzeugeObjekt(m_typUe, m_pidUe, m_nameUe, null);
			m_vdo.erweitereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);

			m_vdo.erzeugeObjekt(m_typUeNachlaufInput, m_pidUeNachlauf1, m_nameUeNachlauf1, null);
			m_vdo.erweitereMenge(m_pidUeNachlauf1, "UrsachenEinheiten", m_anlage);
		}
		catch (Exception e1)
		{
			e1.printStackTrace();
			
      fail("Ursacheneinheit konnte nicht erstellt werden");
		}
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  	m_vdo.reduziereMenge(m_pidUe, "UrsachenEinheiten", m_anlage);
  	m_vdo.loescheObjekt(m_pidUe);

  	m_vdo.reduziereMenge(m_pidUeNachlauf1, "UrsachenEinheiten", m_anlage);
  	m_vdo.loescheObjekt(m_pidUeNachlauf1);

  	m_connection.disconnect(false, "");    
  }

  @Before
  public void setUp() throws Exception
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }

  
  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.26 durchgefhrt.
   */
  @Test
  public final void prueffall_13_26 ()
  {
  	Prueffall prueffall = Prueffall.PRF1;

  	prueffall_13_26_Vorbereitung();
  	prueffall_13_26_Parametrierung(prueffall);
  	prueffall_13_26_Test_1(prueffall);
  	prueffall_13_26_Test_2(prueffall);
  }

  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.27 durchgefhrt.
   */
  @Test
  public final void prueffall_13_27 ()
  {
  	Prueffall prueffall = Prueffall.PRF2;

  	prueffall_13_26_Vorbereitung();
  	prueffall_13_26_Parametrierung(prueffall);
  	prueffall_13_26_Test_1(prueffall);
  	prueffall_13_26_Test_2(prueffall);
  }
  
  /**
   * In diesem Testfall werden die in der Prfspezifikation definierten Testflle
   * fr den Prffall 13.28 durchgefhrt.
   */
  @Test
  public final void prueffall_13_28 ()
  {
  	Prueffall prueffall = Prueffall.PRF3;

  	prueffall_13_26_Vorbereitung();
  	prueffall_13_26_Parametrierung(prueffall);
  	prueffall_13_26_Test_1(prueffall);
  	prueffall_13_26_Test_2(prueffall);
  }

  /**
   * In diesem Testfall wird berprft, ob die fr den JUnitTest notwendige Ursacheneinheit
   * korrekt erstellt wurde und vom Steuerungsalgorithmus bearbeitet wird. berprft wird
   * hierzu der Empfang der Statusinformation "EinzelsituationVorstufe", die nach dem Berechnen
   * versendet wird.
   */
  private void prueffall_13_26_Vorbereitung ()
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);
  	
    EinzelsituationVorstufeEmpfang esve = new EinzelsituationVorstufeEmpfang(m_connection, ue);
    esve.addListener(this);
    
    m_alg.starteBerechnung();
    
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(65000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}

    if (!m_datenErhalten)
			fail ("Vorbereitung EinzelsituationVorstufe nicht empfangen, Ursacheneinheit nicht korrekt erstellt");
  }
  
  /**
   * In diesem Tesfall werden folgende Attribitgruppen der Ursacheneinheit parametriert.<p>
   * - atg.sswSbaEinzelMassnahme<br>
   * - atg.sswSbaUrsachenEinheit<br>
   * - atg.sswSbaUeParameterNachlauf<p>
   */
  private void prueffall_13_26_Parametrierung(Prueffall prueffall)
  {
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

  	SystemObject anzeige = m_dataModel.getObject(m_pidAnzeige);

  	Data atgParam = erzeugeAtgParameterNachlauf(prueffall);  	
  	Data atgEm = erzeugeAtgEinzelMassnahme(erzeugeMassnahmenAnforderung(anzeige));
  	Data atgUe = erzeugeAtgUrsachenEinheit();
  	Data atgMq = null;
  	
		try
		{
			parametriereDynObjUrsachenEinheit(ue, atgParam, atgEm, atgUe, atgMq);
		}
		catch (Exception e)
		{
			fail ("Ursacheneinheit konnte nicht parametriert werden");
		}

  }
  
  /**
   * In dieser Methode wird der Testfall 13.26 (1) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_26_Test_1(Prueffall prueffall)
  {
		m_datenErhalten = false;

		String statusUe1 = "Nein";
		if (prueffall.equals(Prueffall.PRF2))
			statusUe1 = "Ja";
		
		try
		{
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaEinzelsituationVorstufe"), m_dataModel.getAspect("asp.zustand"));

			DynamicObject ueTest1 = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUeNachlauf1);
			SenderOhneStatusRueckmeldung.source(m_connection, ueTest1, dd, erzeugeAtgEinzelsituationVorstufe(statusUe1));											
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		

		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
    		
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
			if (status)
			{
				fail ("Test 1 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
			}
		}		
  }
  

	/**
   * In dieser Methode wird der Testfall 13.26 (2) entsprechend der Prfspezifikation durchgefhrt.<p>
   * Zuerst werden die entsprechenden Daten eingespielt, nach einer kurzen Pause wird die Methode
   * StarteBerechnung des Algorithmus ausgefhrt und auf das Ergebnis gewartet. Nach dem der Status
   * empfangen wurde, wird das Ergebnis berprft.
   */
  private void prueffall_13_26_Test_2(Prueffall prueffall)
  {
		m_datenErhalten = false;

		String statusUe1 = "Ja";
		if (prueffall.equals(Prueffall.PRF2))
			statusUe1 = "Nein";
		
		try
		{
			DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaEinzelsituationVorstufe"), m_dataModel.getAspect("asp.zustand"));

			DynamicObject ueTest1 = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUeNachlauf1);
			SenderOhneStatusRueckmeldung.source(m_connection, ueTest1, dd, erzeugeAtgEinzelsituationVorstufe(statusUe1));											
		}
		catch (SenderException e)
		{
			e.printStackTrace();
		}
		
		m_connection.sleep(2000);
		
		m_alg.starteBerechnung();
		
    try
		{
    	synchronized (m_objekt)
			{
    		m_objekt.wait(10000);				
			}
		}
		catch (InterruptedException e)
		{
			e.printStackTrace();
		}
    		
		if (m_datenErhalten)
		{
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);

			if (prueffall.equals(Prueffall.PRF1))
			{
				if (!status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}
			}
			else if (prueffall.equals(Prueffall.PRF2))
			{
				if (!status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
			else if (prueffall.equals(Prueffall.PRF3))
			{
				if (!status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
		}

		/**
		 * Input Ausschalten
		 */
		if (prueffall.equals(Prueffall.PRF3))
		{
			try
			{
				DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaEinzelsituationVorstufe"), m_dataModel.getAspect("asp.zustand"));

				DynamicObject ueTest1 = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUeNachlauf1);
				SenderOhneStatusRueckmeldung.source(m_connection, ueTest1, dd, erzeugeAtgEinzelsituationVorstufe("Nein"));											
			}
			catch (SenderException e)
			{
				e.printStackTrace();
			}

			m_connection.sleep(2000);
			
			m_alg.starteBerechnung();
			
	    try
			{
	    	synchronized (m_objekt)
				{
	    		m_objekt.wait(10000);				
				}
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
					
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);

			if (status)
			{
				fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
			}			
		}
		else
		{
			/**
			 * Warten, und Status neu bestimmen
			 */
			m_connection.sleep(100000);
	
			m_alg.starteBerechnung();
			
	    try
			{
	    	synchronized (m_objekt)
				{
	    		m_objekt.wait(10000);				
				}
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
					
			boolean status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
	
			if (prueffall.equals(Prueffall.PRF1))
			{
				if (!status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}
			}
			else if (prueffall.equals(Prueffall.PRF2))
			{
				if (!status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
			
			/**
			 * Warten, und Status neu bestimmen
			 */
			m_connection.sleep(25000);
	
			m_alg.starteBerechnung();
			
	    try
			{
	    	synchronized (m_objekt)
				{
	    		m_objekt.wait(10000);				
				}
			}
			catch (InterruptedException e)
			{
				e.printStackTrace();
			}
			
			status = bestimmeStatusEinzelsituationVorstufe();
			System.out.println("Status " + status);
	
			if (prueffall.equals(Prueffall.PRF1))
			{
				if (status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}
			}
			else if (prueffall.equals(Prueffall.PRF2))
			{
				if (status)
				{
					fail ("Test 2 Prffall=" + prueffall + " -> Status der Einzelsituationvorstufe nicht korrekt");				
				}				
			}
		}		
  }

  /**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaEinzelsituationVorstufe</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaEinzelsituationVorstufe</code>.
	 */
	private Data erzeugeAtgEinzelsituationVorstufe(String status)
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaEinzelsituationVorstufe"));
		
		dataNeu.getScaledValue("Status").setText(status);
		dataNeu.getScaledValue("StatusBercksichtigung").setText("automatik");
		dataNeu.getTextValue("Info").setText("JUnitTest");
		
		return dataNeu;
	}
  
	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDaten(de.bsvrz.dav.daf.main.ResultData)
	 */
	@Override
	public void neueDaten(ResultData rdata)
	{
		System.out.println("Abstandswarnung1.neueDaten() " + rdata.getObject().getPid());
		
		m_datenErhalten = true;
		m_DatensatzEinzelsituationVorstufe = rdata;
		
		synchronized (m_objekt)
		{
			m_objekt.notify();					
		}
	}
  
	/**
	 * Bestimmt den aktuellen Zustand der Einzelsituationvorstufe des empfangenen Datensatzes.
	 * 
	 * @return true wenn Einzelsituationvorstufe wahr, ansonsten false.
	 */
	private boolean bestimmeStatusEinzelsituationVorstufe()
	{
		if (m_DatensatzEinzelsituationVorstufe != null)
		{
			if (m_DatensatzEinzelsituationVorstufe.hasData())
			{				
				Data data = m_DatensatzEinzelsituationVorstufe.getData();
				
				return data.getScaledValue("Status").getValueText().equals("Ja");
			}
		}
		
		return false;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaEinzelMassnahme</code>
	 * und gibt ihn zurck.
	 * 
	 * @param massnahmenanforderung Daten der Massnahmeanforderung.
	 * @return Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 */
	private Data erzeugeAtgEinzelMassnahme(Massnahmenanforderung massnahmenanforderung)
	{
		if (massnahmenanforderung != null)
		{				
			Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme"));

			TestTools.fuelleAtgEinzelMassnahme(massnahmenanforderung, dataNeu);
			
			return dataNeu;
		}
		
		return null;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUrsachenEinheit</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 */
	private Data erzeugeAtgUrsachenEinheit()
	{
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit"));
		
		TestTools.fuelleAtgUrsachenEinheit(dataNeu);
		
		return dataNeu;
	}

	/**
	 * Diese Methode erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeAbstandswarnung1</code>
	 * und gibt ihn zurck.
	 * 
	 * @return Datensatz <code>atg.sswSbaUeAbstandswarnung1</code>.
	 */
  private Data erzeugeAtgParameterNachlauf(Prueffall prueffall)
	{
		DataDescription dd = new DataDescription(m_dataModel.getAttributeGroup("atg.sswSbaUeNachlauf"), m_dataModel.getAspect("asp.parameterSoll"));
  	DynamicObject ue = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUe);

  	DynamicObject ueI1 = (DynamicObject)TestTools.getDynObjAusMenge(m_connection, m_anlage, "UrsachenEinheiten", m_pidUeNachlauf1);
  	
		// Aktuelle Parameter holen
		ResultData rdata = m_connection.getData(ue, dd, 1000);
				
		// Wert setzen
		ParameterNachlauf parameter = new ParameterNachlauf(rdata);

		if (prueffall.equals(Prueffall.PRF1))
		{
			parameter.setUrsacheneinheit(ueI1);
			parameter.setReaktion(Reaktion.EINSCHALTUNG);
			parameter.setEinschaltdauer(120000);			
		}
		else if (prueffall.equals(Prueffall.PRF2))
		{
			parameter.setUrsacheneinheit(ueI1);
			parameter.setReaktion(Reaktion.AUSSCHALTUNG);
			parameter.setEinschaltdauer(120000);			
		}
		else if (prueffall.equals(Prueffall.PRF3))
		{
			parameter.setUrsacheneinheit(ueI1);
			parameter.setReaktion(Reaktion.EINSCHALTUNG);
			parameter.setEinschaltdauer(120000);			
		}

		return parameter.erzeugeDatensatz((ClientDavConnection)m_connection);
	}

  /**
   * Methode erzeugt eine Manahmenanforderung mit der fr eine Anzeige ein bestimmter WVZ-Inhalt fr einen bestimmten
   * Schaltgrund angefordert wird
   * @param anzeige Anzeige die geschaltet werden soll
   * @return erzeugte Manahmenanforderung
   */
  private Massnahmenanforderung erzeugeMassnahmenAnforderung(SystemObject anzeige)
  {
    Massnahmenanforderung ma = new Massnahmenanforderung ();
    Vector<EinzelMassnahmeAlternative> einzelMassnahmeAlternative = new Vector<EinzelMassnahmeAlternative>();
    ma.setEinzelMassnahmeAlternative(einzelMassnahmeAlternative);
    
    EinzelMassnahmeAlternative alternative = new EinzelMassnahmeAlternative();
    einzelMassnahmeAlternative.add(alternative);
    
    Vector<AnzeigenDatensatz> datenSaetze = new Vector<AnzeigenDatensatz>();
    alternative.setEinzelMassnahmeAlternative(datenSaetze);
        
    AnzeigenDatensatz datensatz = new AnzeigenDatensatz();
    datenSaetze.add(datensatz);
    
		Data dataNeu = m_connection.createData(m_dataModel.getAttributeGroup("atg.sswSbaMassnahmenAnforderung"));
		dataNeu.getArray("EinzelMassnahmeAlternative").setLength(1);
		dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").setLength(1);
		Data dataWvzInhalt = dataNeu.getArray("EinzelMassnahmeAlternative").getItem(0).getArray("AnzeigenDatenstze").getItem(0).getItem("WVZInhalt"); 

		SystemObject anzeigeInhalt = m_dataModel.getObject("bw.uz.B27.streckenBeeinflussungsAnlage.wvzInhalt.tlsGefahrenstelle");
		
		// WVZInhalt
		dataWvzInhalt.getReferenceValue("AnzeigeInhalt").setSystemObject(anzeigeInhalt);
		dataWvzInhalt.getScaledValue("Blinken").setText("Nein");
		dataWvzInhalt.getScaledValue("Blinktakt").set(-1);
		dataWvzInhalt.getArray("GrafikDarstellungen").setLength(0);
		dataWvzInhalt.getReferenceValue("Wechseltext").setText("undefiniert");
    
    datensatz.setAnzeige(anzeige);
    datensatz.setAnzeigeInhalt(anzeigeInhalt);
    datensatz.setAbsolutNotwendig(false);
    datensatz.setNotwendig(false);
    datensatz.setPrioritt(-1);
    datensatz.setVerschiebbarNachfolger(0);
    datensatz.setVerschiebbarVorgnger(0);
    datensatz.setWvzInhalt(dataWvzInhalt);
    
    return ma;
  }
	/**
	 * Diese Methode parametriert das bergebene Objekt mit den bergebenen Datenstze.
	 * Wird <code>null</code> bergeben wird die entsprechende Attributgruppe nicht
	 * parametriert.
	 * 
	 * @param ue Dyn. Objekt Ursacheneinheit.
	 * @param atgParam Datensatz <code>atg.sswSbaUeNachlauf</code>.
	 * @param atgEm Datensatz <code>atg.sswSbaEinzelMassnahme</code>.
	 * @param atgUe Datensatz <code>atg.sswSbaUrsachenEinheit</code>.
	 * @param atgmq Datensatz <code>atg.sswSbaUeMqZuordnung</code>.
	 */
	private void parametriereDynObjUrsachenEinheit(DynamicObject ue, Data atgParam, Data atgEm, Data atgUe, Data atgMq)
	{
		if (ue != null)
		{
			System.out.println("Ursacheneinheit " + ue + " parametrieren");
			
			Aspect asp = m_dataModel.getAspect("asp.parameterVorgabe");

      if (atgParam != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeNachlauf");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgParam, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

      if (atgEm != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaEinzelMassnahme");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgEm, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }
      
      if (atgUe != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUrsachenEinheit");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgUe, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

      if (atgMq != null)
      {
				try
				{
					AttributeGroup atg = m_dataModel.getAttributeGroup("atg.sswSbaUeMqZuordnung");
					DataDescription dd = new DataDescription(atg, asp);

					SimpleSenderMitWartenAufParametrierung.sendWait(m_connection, ue, dd, atgMq, 60000);
				}
				catch (SenderException e)
				{
					e.printStackTrace();
				} // Sende als Sender
      }

		}			
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.INeueDatenListener#neueDatenSituationAnalyseMq(de.bsvrz.ste.algorithmusrahmen.algorithmen.lib.SituationAnalyseMq)
	 */
	@Override
	public void neueDatenSituationAnalyseMq(SituationAnalyseMq analyse)
	{
	}

	@Override
	public void neueDatenStaukriterium(StaukriteriumErgebnis staukriterium)
	{
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.ste.stealgsba.algorithmen.lib.IUfdsNeueDatenListener#neueDatenSituationAnalyseUfds(de.bsvrz.ste.stealgsba.algorithmen.lib.SituationAnalyseUfd)
	 */
	@Override
	public void neueDatenSituationAnalyseUfds(SituationAnalyseUfd analyse)
	{
		System.out.println("Nachlauf.neueDatenSituationAnalyseMq()" + analyse);

		synchronized (m_objekt2)
		{
			m_objekt2.notify();					
		}		
	}

}
