/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.nachlauf;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Diese Klasse speichert die Daten der Attributgruppe !atg.sswSbaUeNachlauf!.<p>
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.3 $ / $Date: 2015/09/10 11:32:59 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class ParameterNachlauf
{
	/**
	 * Pid der Attributgruppe.
	 */
	private String m_pidAtg = "atg.sswSbaUeNachlauf";
	/**
	 * Referenz auf eine Ursacheneinheit.
	 */
	private SystemObject m_ursacheneinheit;
	/**
	 * Reaktion der Ursacheneinheit Ein-/Ausschaltung.
	 */
	private Reaktion m_reaktion;
	/**
	 * Einschaltdauer in Millisekunden.
	 */
	private long m_einschaltdauer;
	
	
	public enum Reaktion
	{
		AUSSCHALTUNG,
		EINSCHALTUNG;
		
		public static Reaktion getWithDavWert(int wertDav)
		{
			if (wertDav == 0)
			{
				return AUSSCHALTUNG;				
			}
			else if (wertDav == 1)
			{
				return EINSCHALTUNG;
			}
			
			return AUSSCHALTUNG;
		}
		
		public static String getText(Reaktion wert)
		{
			if (wert != null)
			{
				if (wert.equals(AUSSCHALTUNG))
				{
					return "Ausschaltung";
				}
				else if (wert.equals(EINSCHALTUNG))
				{
					return "Einschaltung";
				}
			}
			
			return "Ausschaltung";
		}
	}
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public ParameterNachlauf()
	{
	}
	/**
	 * Konstruktor mit bernahme des bergebenen Datensatzes in die interne Struktur.
	 *  
	 * @param rdata Datensatz der Parameter.
	 */
	public ParameterNachlauf(ResultData rdata)
	{
		setdaten(rdata);
	}
	
	/**
	 * bernahme des bergebenen Datensatzes in die interne Struktur.
	 * 
	 * @param rdata Datensatz der Parameter.
	 */
	public void setdaten(ResultData rdata)
	{
		if (rdata != null)
		{
			Data data = rdata.getData();
			if (data != null)
			{
				m_ursacheneinheit = data.getReferenceValue("Ursacheneinheit").getSystemObject();
				m_reaktion = Reaktion.getWithDavWert(data.getUnscaledValue("Reaktion").intValue());
				m_einschaltdauer = data.getTimeValue("Einschaltdauer").getMillis();

			}
			else
			{
				m_ursacheneinheit = null;
				m_reaktion = Reaktion.AUSSCHALTUNG;
				m_einschaltdauer = 0;
			}
		}		
	}

	/**
	 * Erzeugt einen Datensatz der Attributgruppe <code>atg.sswSbaUeHarmoGeschwDiff</code> mit
	 * den Daten der internen Struktur.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @return erzeugter Datensatz.
	 */
	public Data erzeugeDatensatz(ClientDavConnection connection)
	{
		DataModel dm = connection.getDataModel();		
		Data dataNeu = connection.createData(dm.getAttributeGroup(m_pidAtg));

		if (dataNeu != null)
		{	
			dataNeu.getReferenceValue("Ursacheneinheit").setSystemObject(getUrsacheneinheit());			
			dataNeu.getUnscaledValue("Reaktion").set(getReaktion().ordinal());
			dataNeu.getTimeValue("Einschaltdauer").setMillis(getEinschaltdauer());
			
			return dataNeu;
		}
		
		return null;
	}
	/**
	 * Gibt den Wert der Variable m_ursacheneinheit zurck.
	 *
	 * @return Wert der Variable m_ursacheneinheit.
	 */
	public SystemObject getUrsacheneinheit()
	{
		return m_ursacheneinheit;
	}
	/**
	 * Setzt den Wert der Variable m_ursacheneinheit.
	 *
	 * @param ursacheneinheit zu setzender Wert der Variable m_ursacheneinheit.
	 */
	public void setUrsacheneinheit(SystemObject ursacheneinheit)
	{
		m_ursacheneinheit = ursacheneinheit;
	}
	/**
	 * Gibt den Wert der Variable m_reaktion zurck.
	 *
	 * @return Wert der Variable m_reaktion.
	 */
	public Reaktion getReaktion()
	{
		return m_reaktion;
	}
	/**
	 * Setzt den Wert der Variable m_reaktion.
	 *
	 * @param reaktion zu setzender Wert der Variable m_reaktion.
	 */
	public void setReaktion(Reaktion reaktion)
	{
		m_reaktion = reaktion;
	}
	/**
	 * Gibt den Wert der Variable m_einschaltdauer in Millisekunden zurck.
	 *
	 * @return Wert der Variable m_einschaltdauer in Millisekunden.
	 */
	public long getEinschaltdauer()
	{
		return m_einschaltdauer;
	}
	/**
	 * Setzt den Wert der Variable m_einschaltdauer.
	 *
	 * @param einschaltdauer zu setzender Wert der Variable m_einschaltdauer.
	 */
	public void setEinschaltdauer(long einschaltdauer)
	{
		m_einschaltdauer = einschaltdauer;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "ParameterNachlauf [m_ursacheneinheit=" + m_ursacheneinheit + ", m_reaktion=" + m_reaktion + ", m_einschaltdauer=" + m_einschaltdauer + "]";
	}
}